<?php 

/**
 Audiophonics Setup Controller
 @Copyright 2017 Stefan Rick
 @author Stefan Rick
 Mail: stefan@rick-software.de
 Web: http://www.max2play.com

 */

include_once(APPLICATION_PATH.'/controller/Start.php');

class Audiophonics_Setup extends Start {

	public $scriptPath = '';
	
	public $usbSoundCards = array('' => array('dtoverlay' => '', 'name' => 'none', 'squeezelite' => '-o default:CARD=ALSA -a 120:::', 'shairport' => '-d default:CARD=ALSA', 'mpd_mixer_control' => 'PCM'),
			'Audiophonics-I-Sabre-V3' => array('dtoverlay' => 'hifiberry-dac', 'name' => 'Audiophonics I-Sabre-V3', 'squeezelite' => '-o sysdefault:CARD=sndrpihifiberry -a 80:4::', 'shairport' => '-d sysdefault:CARD=sndrpihifiberry',  'mpd_device' => 'default:CARD=sndrpihifiberry','mpd_mixer_control' => 'Digital', 'amixercard' => 'sndrpihifiberry')
			);		
	public $alsasettings = array(
			// No Settings for Current DAC driver
			);
	
	public function __construct(){				
		parent::__construct();
		$this->pluginname = _('Audiophonics');
		$this->registerLocale(dirname(__FILE__).'/../locale', 'audiophonics');
		$this->scriptPath = dirname(__FILE__).'/../scripts/';				
		
		if($_REQUEST['ajax'] == 1){
			if ($_REQUEST['action'] == 'display_install')
				$this->_installDisplay($ajax = 1);
			ob_end_clean();
			echo implode('<br />', $this->view->message);
			ob_flush();
			die();
		}
		
		$this->checkPicturesAndHeader('audiophonics', dirname(__FILE__));			
				
		// alsamixerParameter is used to get % values instead of -db (linear scale instead of exponential volume change)
		$this->alsamixerParameter = '-M';
				
		$this->_getDTOverlayConfig();
		
		if(isset($_REQUEST['action'])){
			if($_REQUEST['action'] == 'save_dtoverlay'){
				$this->_saveDtoverlay();
				$this->_getDTOverlayConfig();
			}
			if($_REQUEST['action'] == 'save_dtoverlay'){
				$this->_saveAlsa();
			}			
		}
		$this->getAlsaAudio();
		$this->_powerButtonHandler();
		$this->_displayHandler();
		$this->easyinstaller = true;		
	}
	
	private function _getDTOverlayConfig(){
		$this->view->dtoverlay = $this->getConfigFileParameter('/opt/max2play/options.conf', 'audiocard');
		if($this->view->dtoverlay == ''){
			$this->view->message[] = _t('You have not yet choosen your installed Audiophonics audiocard! You must do this before you can use your audioplayers with your Audiophonics card! Please choose your card and finish the setup below.');
		}
		return true;
	}
	
	private function _saveDtoverlay(){
		if(isset($_REQUEST['dtoverlay']) && in_array($_REQUEST['dtoverlay'], array_keys($this->usbSoundCards)) && $this->view->dtoverlay != $_REQUEST['dtoverlay']){										
			$this->saveConfigFileParameter('/boot/config.txt', 'dtoverlay', $this->usbSoundCards[$_REQUEST['dtoverlay']]['dtoverlay']);
			$this->saveConfigFileParameter('/opt/max2play/options.conf', 'audiocard', $_REQUEST['dtoverlay']);
			//Set Squeezelite Parameter
			$this->saveConfigFileParameter('/opt/max2play/audioplayer.conf', 'SQUEEZELITE_PARAMETER', $this->usbSoundCards[$_REQUEST['dtoverlay']]['squeezelite']);
			$this->saveConfigFileParameter('/opt/max2play/audioplayer.conf', 'SHAIRPORT_PARAMETER', $this->usbSoundCards[$_REQUEST['dtoverlay']]['shairport']);			
			
			//MPD Parameter Justboom
			if(file_exists('/etc/mpd.conf')){
				$this->saveConfigFileParameter('/etc/mpd.conf', $parameter = 'mixer_control', $value = '\"'.$this->usbSoundCards[$_REQUEST['dtoverlay']]['mpd_mixer_control'].'\"', $separator = '\\t');
				$this->saveConfigFileParameter('/etc/mpd.conf', $parameter = 'format', $value = '\"192000:24:2\"', $separator = '\\t');
				$this->saveConfigFileParameter('/etc/mpd.conf', $parameter = 'device', $value = '\"'.$this->usbSoundCards[$_REQUEST['dtoverlay']]['mpd_device'].'\"', $separator = '\\t');
				$this->saveConfigFileParameter('/etc/mpd.conf', $parameter = 'mixer_device', $value = '\"'.$this->getSoundcardPosition('sndrpijustboomd').'\"', $separator = '\\t');
			}						
			
			$this->view->message[] = _("Boot config parameters changed");
			$this->view->message[] = str_replace('$SERVERIP', $this->getServerUrl($ipv4=true), _('Please restart your device! To restart now <a href="http://$SERVERIP/plugins/max2play_settings/controller/Basic.php?action=reboot&redirecturl='.urlencode('http://').'$SERVERIP">just click here</a>.'));
		}		
		return true;
	}
	
	/**
	 * get soundcard position e.g. hw:0 or hw:1
	 * @param string $soundcard
	 * @return string for MPD mixer_device
	 */
	private function getSoundcardPosition($soundcard = ''){
		$aplayOutput = $this->writeDynamicScript(array('aplay -l | grep "card"'));
		//card 0: ALSA [bcm2835 ALSA], device 1: bcm2835 ALSA [bcm2835 IEC958/HDMI]
		//card 1: IQaudIODAC [IQaudIODAC], device 0: IQaudIO DAC HiFi pcm512x-hifi-0 []
		if(!preg_match_all('=^card ([0-9]): ([^ ]*)=im', $aplayOutput, $aplayMatches))
			return 'default';
		$count = count($aplayMatches[1]);
		$aplayCards = array();
		for($i = 0; $i < $count; $i++){
			$aplayCards[$aplayMatches[2][$i]] = $aplayMatches[1][$i];
		}
		if(isset($aplayCards[$soundcard]))
			return 'hw:'.$aplayCards[$soundcard];
		else
			return 'default';
	}	
	
	/**
	 * get Alsa Settings For AudioCard
	 */
	public function getAlsaAudio(){
		$this->view->alsavalues = false;
		if($this->view->dtoverlay && isset($this->alsasettings[$this->view->dtoverlay])){
			$user = $this->getSystemUser();
			if(isset($this->usbSoundCards[$this->view->dtoverlay]['amixercard'])){
				$amixercard = ' -c '.$this->usbSoundCards[$this->view->dtoverlay]['amixercard'].' ';
			}else{
				$amixercard = '';
			}
			foreach($this->alsasettings[$this->view->dtoverlay] as $key => $value){				
				$script = array('su - '.$user.' -c \'amixer '.$this->alsamixerParameter.' '.$amixercard.' sget "'.$key.'"\'');
				$output = $this->writeDynamicScript($script);
				preg_match('=\[([^\]]*)\]=', $output, $match);
				if(isset($match[1])){
					$this->view->alsavalues[$key] = $value;
					if(strpos($match[1], '%') !== FALSE)
						$this->view->alsavalues[$key]['type'] = '%';
					else
						$this->view->alsavalues[$key]['type'] = '';
					$this->view->alsavalues[$key]['value'] = intval($match[1]);
				}
			}
		}		
		return true;
	}
	
	public function _saveAlsa(){
		$user = $this->getSystemUser();
		$this->getAlsaAudio();
		$script = array();
		if(isset($this->usbSoundCards[$this->view->dtoverlay]['amixercard'])){
			$amixercard = ' -c '.$this->usbSoundCards[$this->view->dtoverlay]['amixercard'].' ';
		}else{
			$amixercard = '';
		}		
		foreach($this->view->alsavalues as $key => $value){
			$script[] = 'su - '.$user.' -c \'amixer '.$this->alsamixerParameter.' '.$amixercard.' -q set "'.$key.'" '.$_REQUEST[str_replace(' ','_', $key)].$value['type'].'\'';
			$script[] = 'su - '.$user.' -c \'alsactl store\'';
		}		
		if(isset($script[0])){
			$output = nl2br($this->writeDynamicScript($script));
			if($output != '')
				$this->view->message[] = $output;
			$this->view->message[] = _("Updated Alsa Settings");
		}
		return true;
	}
	
	public function _autostartPowerButton(){
		$this->selectAutostart(isset($_REQUEST['autostartpowerbutton']) ? 1 : 0, true, 'PWRBUTTON_AUTOSTART');
		return true;
	}
	
	public function _setupPowerButton(){
		$this->selectAutostart(1, true, 'PWRBUTTON_AUTOSTART');
		// Add script to rc.local
		$this->view->message[] = nl2br($this->writeDynamicScript(array($this->scriptPath .'install_powerbutton.sh')));
		return true;
	}
	
	public function _powerButtonHandler(){
		if(isset($_REQUEST['action']) && $_REQUEST['action'] == 'pwrbutton_install'){
			$this->_setupPowerButton();
		}
		if(isset($_REQUEST['action']) && $_REQUEST['action'] == 'pwrbutton_save'){
			$this->_autostartPowerButton();			
			$this->view->message[] = str_replace('$SERVERIP', $this->getServerUrl($ipv4=true), _('Please restart your device! To restart now <a href="http://$SERVERIP/plugins/max2play_settings/controller/Basic.php?action=reboot&redirecturl='.urlencode('http://').'$SERVERIP">just click here</a>.'));
		}
		// Check for running Script
		if(strpos(file_get_contents('/etc/rc.local'), 'PWRBUTTON_AUTOSTART') == FALSE){
			$this->pwrbutton_installed = false;
			return false;
		}
		$this->pwrbutton_installed = true;
		$this->pwrButtonPID = $this->status("pwrbutton");		
		$this->autostartpowerbutton = $this->checkAutostart('PWRBUTTON_AUTOSTART', true);
		return true;
	}
	
	public function _configureDisplay(){
		
		// TODO: Edit Values for LMS, PLAYER, PURPOSE, WINSTAR OLED 
				
		return true;
	}
	
	public function _displayHandler(){
		if(isset($_REQUEST['action']) && $_REQUEST['action'] == 'display_install'){
			$this->selectAutostart(1, true, 'DISPLAY_AUTOSTART');
			$this->_installDisplay();
		}
		if(isset($_REQUEST['action']) && $_REQUEST['action'] == 'display_save'){
			$this->selectAutostart(isset($_REQUEST['autostartdisplay']) ? 1 : 0, true, 'DISPLAY_AUTOSTART');
			$this->configureDisplay();
			$this->view->message[] = str_replace('$SERVERIP', $this->getServerUrl($ipv4=true), _('Please restart your device! To restart now <a href="http://$SERVERIP/plugins/max2play_settings/controller/Basic.php?action=reboot&redirecturl='.urlencode('http://').'$SERVERIP">just click here</a>.'));
		}
		// Check for existing Script
		if(!file_exists('/opt/Raspdac-Display/')){
			$this->display_installed = false;
			return false;
		}
		$this->display_installed = true;
		$this->displayPID = $this->status("python");
		$this->autostartdisplay = $this->checkAutostart('DISPLAY_AUTOSTART', true);
		return true;
	}
	
	/**
	 * Install Display Support for Winstar OLED on Raspberry Pi
	 * @param $ajax IsAjax
	 * @return boolean
	 */
	private function _installDisplay($ajax = 0){
		$outfile = '/opt/max2play/cache/installDisplay.txt';
		if($ajax == 0){
			ignore_user_abort(true);
			set_time_limit(3000);
			$this->view->message[] = _t('Display Setup Started');
			if($this->getProgressWithAjax($outfile, 1, 0)){
				//run it as a deamon to enable apache updates
				$shellanswer = $this->writeDynamicScript(array($this->scriptPath ."install_display.sh >> ".$outfile." 2>&1 &", false, true));
			}
		}else{
			$status = $this->getProgressWithAjax($outfile, 0, 0, 30);
			$this->view->message[] = nl2br($status);
			if(strpos($status, 'Finished Display Installer') !== FALSE){
				//Finished Progress - did not delete progressfile
				shell_exec('rm '.$outfile);
			}
		}
		return true;
	}
	
	public function showHelpSidebar(){
		global $helpSidebar;
		$helpSidebar['title'] = _t('Help - Audiophonics Edition');
		$helpSidebar['content'] = _t('<ul><li>Choose your setup and easily configure your audiosettings with this special Max2Play-Version. For video tutorials and more about Max2Play visit www.max2play.com</li></ul>');
		$helpSidebar['toppicture'] = '/images/start-toppicture.jpg';
		return true;
	}	
		
}

$start = new Audiophonics_Setup();

include_once(dirname(__FILE__).'/../view/setup.php');

