#!/bin/bash

CURRENTDIR="$( cd "$( dirname "${BASH_SOURCE[0]}" )" && pwd )"

apt-get update

# get Raspdac
pushd /opt/

if [ -e /opt/Raspdac-Display ]; then
	echo "Delete Existing RaspDac-Display Folder"
	rm -R /opt/Raspdac-Display
fi
git clone https://github.com/dhrone/Raspdac-Display
pushd /opt/Raspdac-Display

# Add the necessary packages to support the RaspDac-Display program

apt-get install python-setuptools python-pip python-dev -y --force-yes
pip install moment
pip install pylms
pip install python-mpd2
pip install arrow
pip install regex
pip install ruamel.yaml


# Set default values in RaspDacDisplay.py
# get purpose value to activate LMS / MPD
# Disable Rune!

PURPOSE=$(grep -a "purpose" /opt/max2play/options.conf | sed -n -e 's/^[A-Z_]*\=//p')
if [ "squeeze" == "$PURPOSE" -o "" == "$PURPOSE" ]; then
   # set LMS - Defaults to localhost
   if [ ! -e /opt/max2play/discover_squeezebox_server ]; then
      echo "<b>ERROR: Squeezebox Server Discovery Script not found! Update Max2Play to latest Version (or Beta)</b>"
   fi
   LMS_SERVER=$(/opt/max2play/discover_squeezebox_server | grep SERVER | sed 's/SERVER://')
   if [ ! -e "$SERVER" ]; then
      sed -i "s/^LMS_SERVER = .*/LMS_SERVER = \"$LMS_SERVER\"/" RaspDacDisplay.py
      echo "Set SERVER ADDRESS to $LMS_SERVER"
   fi
   
   ETH0=$(cat /sys/class/net/eth0/operstate)
   if [ "$ETH0" = "up" ]; then
	    INTERFACE=eth0
   else
	    INTERFACE=wlan0
   fi
   if [ -e /sys/class/net/$INTERFACE/address ]; then
	  MAC="$(cat /sys/class/net/$INTERFACE/address)"
	  echo "Use Interface $INTERFACE with $MAC"
   else
	  MAC=""
   fi
   
   if [ ! -e "$MAC" ]; then
        sed -i "s/^LMS_PLAYER = .*/LMS_PLAYER = \"$MAC\"/" RaspDacDisplay.py
        echo "Set MAC ADDRESS to $MAC"
   fi
   
   sed -i 's/^LMS_ENABLED = .*/LMS_ENABLED = True/' RaspDacDisplay.py 
   echo "Activated Squeezebox Server in RaspDAC-Display"
fi

if [ "mpd" == "$PURPOSE" ]; then
   sed -i 's/^MPD_ENABLED = .*/MPD_ENABLED = true/' RaspDacDisplay.py 
fi

#LMS_PLAYER (set this to the MAC address for your Max2Play system if using SqueezeLite).


# TODO: Edit the Winstar_GraphicOLED.py file and change the pin settings for the version of the Raspdac that you have.

if [ ! "$(grep -a 'DISPLAY_AUTOSTART' /etc/rc.local)" ]; then
	sed -i "s@^exit 0@if [ \"\$(grep -a 'DISPLAY_AUTOSTART=1' /opt/max2play/autostart.conf)\" ]; then \n  /usr/bin/python2 /opt/Raspdac-Display/RaspDacDisplay.py \&\nfi\nexit 0@" /etc/rc.local
fi

# Get current Timezone
CURRTIME=$(cat /etc/timezone)
echo "Set Timezone to $CURRTIME"
sed -i "s@^TIMEZONE=.*@TIMEZONE=\"$CURRTIME\"@" RaspDacDisplay.py

echo "Finished Display Installer - Reboot needed: <b><a href='/plugins/max2play_settings/controller/Basic.php?action=reboot&redirecturl=/plugins/audiophonics/controller/Setup.php'>Click here to Restart Device</a></b>";

exit 0
